'use strict';

const shared = require('@vueuse/shared');
const vue = require('vue');
const unhead = require('unhead');
const dom = require('@unhead/dom');

function resolveUnrefHeadInput(ref) {
  const root = shared.resolveUnref(ref);
  if (!ref || !root)
    return root;
  if (Array.isArray(root))
    return root.map(resolveUnrefHeadInput);
  if (typeof root === "object") {
    return Object.fromEntries(
      Object.entries(root).map(([key, value]) => {
        if (key === "titleTemplate")
          return [key, vue.unref(value)];
        return [key, resolveUnrefHeadInput(value)];
      })
    );
  }
  return root;
}
function asArray(value) {
  return Array.isArray(value) ? value : [value];
}

const VueTriggerDomPatchingOnUpdatesPlugin = (options) => {
  return unhead.defineHeadPlugin({
    hooks: {
      "entries:updated": function(head) {
        dom.debouncedRenderDOMHead(vue.nextTick, head, { document: options?.document });
      }
    }
  });
};

const VueReactiveInputPlugin = () => {
  return unhead.defineHeadPlugin({
    hooks: {
      "entries:resolve": function(ctx) {
        for (const entry of ctx.entries)
          entry.input = resolveUnrefHeadInput(entry.input);
      }
    }
  });
};

const Vue3 = vue.version.startsWith("3");
vue.version.startsWith("2.");
const IsBrowser = typeof window !== "undefined";

function useHead$2(input, options = {}) {
  const head = injectHead();
  head.push(input, options);
}

function useServerHead$1(input, options = {}) {
  useHead$2(input, { ...options, mode: "server" });
}

function useHead$1(input, options = {}) {
  const head = injectHead();
  const vm = vue.getCurrentInstance();
  if (!vm) {
    head.push(input, options);
    return;
  }
  const resolvedInput = vue.ref({});
  vue.watchEffect(() => {
    resolvedInput.value = resolveUnrefHeadInput(input);
  });
  let entry;
  vue.watch(resolvedInput, (e) => {
    if (!entry)
      entry = head.push(e, options);
    else
      entry.patch(e);
  }, { immediate: true });
  vue.onBeforeUnmount(() => {
    entry?.dispose();
  });
}

function useServerHead(input, options = {}) {
  if (!IsBrowser)
    useServerHead$1(input, options);
}
function useHead(input, options = {}) {
  if (options.mode === "server" && IsBrowser || options.mode === "client" && !IsBrowser)
    return;
  IsBrowser ? useHead$1(input, options) : useHead$2(input, options);
}
const useTitle = (title) => useHead({ title });
const useTitleTemplate = (titleTemplate) => useHead({ titleTemplate });
const useMeta = (meta) => useHead({ meta: asArray(meta) });
const useLink = (link) => useHead({ link: asArray(link) });
const useScript = (script) => useHead({ script: asArray(script) });
const useStyle = (style) => useHead({ style: asArray(style) });
const useNoscript = (noscript) => useHead({ noscript: asArray(noscript) });
const useBase = (base) => useHead({ base });
const useHtmlAttrs = (attrs) => useHead({ htmlAttrs: attrs });
const useBodyAttrs = (attrs) => useHead({ bodyAttrs: attrs });

const headSymbol = Symbol("unhead");
function injectHead() {
  return vue.getCurrentInstance() && vue.inject(headSymbol) || unhead.getActiveHead();
}
function createHead(options = {}) {
  const plugins = [
    unhead.HydratesStatePlugin(),
    VueReactiveInputPlugin(),
    ...options?.plugins || []
  ];
  const head = unhead.createHead({
    ...options,
    plugins
  });
  const vuePlugin = {
    install(app) {
      if (Vue3)
        app.config.globalProperties.$head = head;
      app.provide(headSymbol, head);
      app.mixin({
        created() {
          const instance = vue.getCurrentInstance();
          if (!instance)
            return;
          const options2 = instance.type;
          if (!options2 || !("head" in options2))
            return;
          const source = typeof options2.head === "function" ? () => options2.head() : options2.head;
          useHead(source);
        }
      });
    }
  };
  return { ...vuePlugin, ...head };
}

function createDomHead(options = {}) {
  return createHead({
    ...options,
    plugins: [
      VueTriggerDomPatchingOnUpdatesPlugin({ document: options?.document }),
      ...options.plugins || []
    ]
  });
}

const HeadVuePlugin = function(_Vue) {
  _Vue.mixin({
    beforeCreate() {
      const options = this.$options;
      if (options.head) {
        const origProvide = options.provide;
        options.provide = function() {
          let origProvideResult;
          if (typeof origProvide === "function")
            origProvideResult = origProvide.call(this);
          else
            origProvideResult = origProvide || {};
          return {
            ...origProvideResult,
            [headSymbol]: options.head
          };
        };
        if (!this.$head)
          this.$head = options.head;
      } else if (!this.$head && options.parent && options.parent.$head) {
        this.$head = options.parent.$head;
      }
    }
  });
};

exports.HeadVuePlugin = HeadVuePlugin;
exports.VueReactiveInputPlugin = VueReactiveInputPlugin;
exports.VueTriggerDomPatchingOnUpdatesPlugin = VueTriggerDomPatchingOnUpdatesPlugin;
exports.asArray = asArray;
exports.createDomHead = createDomHead;
exports.createHead = createHead;
exports.headSymbol = headSymbol;
exports.injectHead = injectHead;
exports.resolveUnrefHeadInput = resolveUnrefHeadInput;
exports.useBase = useBase;
exports.useBodyAttrs = useBodyAttrs;
exports.useHead = useHead;
exports.useHtmlAttrs = useHtmlAttrs;
exports.useLink = useLink;
exports.useMeta = useMeta;
exports.useNoscript = useNoscript;
exports.useScript = useScript;
exports.useServerHead = useServerHead;
exports.useStyle = useStyle;
exports.useTitle = useTitle;
exports.useTitleTemplate = useTitleTemplate;
for (const k in dom) {
  if (k !== 'default' && !exports.hasOwnProperty(k)) exports[k] = dom[k];
}
