import { resolveUnref } from '@vueuse/shared';
import { unref, nextTick, version, getCurrentInstance, ref, watchEffect, watch, onBeforeUnmount, inject } from 'vue';
import { defineHeadPlugin, getActiveHead, HydratesStatePlugin, createHead as createHead$1 } from 'unhead';
import { debouncedRenderDOMHead } from '@unhead/dom';
export * from '@unhead/dom';

function resolveUnrefHeadInput(ref) {
  const root = resolveUnref(ref);
  if (!ref || !root)
    return root;
  if (Array.isArray(root))
    return root.map(resolveUnrefHeadInput);
  if (typeof root === "object") {
    return Object.fromEntries(
      Object.entries(root).map(([key, value]) => {
        if (key === "titleTemplate")
          return [key, unref(value)];
        return [key, resolveUnrefHeadInput(value)];
      })
    );
  }
  return root;
}
function asArray(value) {
  return Array.isArray(value) ? value : [value];
}

const VueTriggerDomPatchingOnUpdatesPlugin = (options) => {
  return defineHeadPlugin({
    hooks: {
      "entries:updated": function(head) {
        debouncedRenderDOMHead(nextTick, head, { document: options?.document });
      }
    }
  });
};

const VueReactiveInputPlugin = () => {
  return defineHeadPlugin({
    hooks: {
      "entries:resolve": function(ctx) {
        for (const entry of ctx.entries)
          entry.input = resolveUnrefHeadInput(entry.input);
      }
    }
  });
};

const Vue3 = version.startsWith("3");
version.startsWith("2.");
const IsBrowser = typeof window !== "undefined";

function useHead$2(input, options = {}) {
  const head = injectHead();
  head.push(input, options);
}

function useServerHead$1(input, options = {}) {
  useHead$2(input, { ...options, mode: "server" });
}

function useHead$1(input, options = {}) {
  const head = injectHead();
  const vm = getCurrentInstance();
  if (!vm) {
    head.push(input, options);
    return;
  }
  const resolvedInput = ref({});
  watchEffect(() => {
    resolvedInput.value = resolveUnrefHeadInput(input);
  });
  let entry;
  watch(resolvedInput, (e) => {
    if (!entry)
      entry = head.push(e, options);
    else
      entry.patch(e);
  }, { immediate: true });
  onBeforeUnmount(() => {
    entry?.dispose();
  });
}

function useServerHead(input, options = {}) {
  if (!IsBrowser)
    useServerHead$1(input, options);
}
function useHead(input, options = {}) {
  if (options.mode === "server" && IsBrowser || options.mode === "client" && !IsBrowser)
    return;
  IsBrowser ? useHead$1(input, options) : useHead$2(input, options);
}
const useTitle = (title) => useHead({ title });
const useTitleTemplate = (titleTemplate) => useHead({ titleTemplate });
const useMeta = (meta) => useHead({ meta: asArray(meta) });
const useLink = (link) => useHead({ link: asArray(link) });
const useScript = (script) => useHead({ script: asArray(script) });
const useStyle = (style) => useHead({ style: asArray(style) });
const useNoscript = (noscript) => useHead({ noscript: asArray(noscript) });
const useBase = (base) => useHead({ base });
const useHtmlAttrs = (attrs) => useHead({ htmlAttrs: attrs });
const useBodyAttrs = (attrs) => useHead({ bodyAttrs: attrs });

const headSymbol = Symbol("unhead");
function injectHead() {
  return getCurrentInstance() && inject(headSymbol) || getActiveHead();
}
function createHead(options = {}) {
  const plugins = [
    HydratesStatePlugin(),
    VueReactiveInputPlugin(),
    ...options?.plugins || []
  ];
  const head = createHead$1({
    ...options,
    plugins
  });
  const vuePlugin = {
    install(app) {
      if (Vue3)
        app.config.globalProperties.$head = head;
      app.provide(headSymbol, head);
      app.mixin({
        created() {
          const instance = getCurrentInstance();
          if (!instance)
            return;
          const options2 = instance.type;
          if (!options2 || !("head" in options2))
            return;
          const source = typeof options2.head === "function" ? () => options2.head() : options2.head;
          useHead(source);
        }
      });
    }
  };
  return { ...vuePlugin, ...head };
}

function createDomHead(options = {}) {
  return createHead({
    ...options,
    plugins: [
      VueTriggerDomPatchingOnUpdatesPlugin({ document: options?.document }),
      ...options.plugins || []
    ]
  });
}

const HeadVuePlugin = function(_Vue) {
  _Vue.mixin({
    beforeCreate() {
      const options = this.$options;
      if (options.head) {
        const origProvide = options.provide;
        options.provide = function() {
          let origProvideResult;
          if (typeof origProvide === "function")
            origProvideResult = origProvide.call(this);
          else
            origProvideResult = origProvide || {};
          return {
            ...origProvideResult,
            [headSymbol]: options.head
          };
        };
        if (!this.$head)
          this.$head = options.head;
      } else if (!this.$head && options.parent && options.parent.$head) {
        this.$head = options.parent.$head;
      }
    }
  });
};

export { HeadVuePlugin, VueReactiveInputPlugin, VueTriggerDomPatchingOnUpdatesPlugin, asArray, createDomHead, createHead, headSymbol, injectHead, resolveUnrefHeadInput, useBase, useBodyAttrs, useHead, useHtmlAttrs, useLink, useMeta, useNoscript, useScript, useServerHead, useStyle, useTitle, useTitleTemplate };
